@@ -23,8 +23,8 @@ contributed by Don Lafontaine <lafont02@cn.ca>
 #include "logger.h"
 #include "ffget.h"
 
-#define FFGET_VERSION	"1.0.0.3"
-#define FFGET_LASTUPDATED "12/11/2002"
+#define FFGET_VERSION	"1.0.0.4"
+#define FFGET_LASTUPDATED "29/03/2006"
 
 #define FFGET_DNORMAL   ((FFGET_debug >= _FFGET_DEBUG_NORMAL  ))
 #define FFGET_DPEDANTIC ((FFGET_debug >= _FFGET_DEBUG_PEDANTIC))
@@ -98,27 +98,25 @@ int FFGET_getnewblock( FFGET_FILE *f )
 		f->startpoint = f->buffer +1;
 		f->FFEOF = 1;
 		return 0;
-	}
-	else
-	    {
 
+	} else {
+		long block_pos;
+
+		block_pos = ftell(f->f);
 		bs = fread( f->buffer, 1, _FFGET_BUFFER_MAX -2, f->f );
 
-		if (bs < (_FFGET_BUFFER_MAX -2))
-		{
-			if (feof(f->f))
-			{
+		if (bs < (_FFGET_BUFFER_MAX -2)) {
+			if (feof(f->f)) {
 				f->FILEEND = 1;
-			}
-			else
-			{
+
+			} else {
+
 				fprintf(stderr,"%s:%d: WARNING - File read failed with error:%s",__FILE__,__LINE__,strerror(errno));
 				return 0;
 			}
 		}
 
-		if (bs > 0)
-		{
+		if (bs > 0) {
 
 			// If we read in some data, then adjust the buffer to deal with it
 		//
@@ -128,6 +126,7 @@ int FFGET_getnewblock( FFGET_FILE *f )
 		// though it has no /real/ purpose)
 		//
 
+			f->last_block_read_from = block_pos;
 			f->startpoint = f->buffer;
 			f->endpoint = f->startpoint +bs -1;
 			f->bytes += bs;
@@ -199,6 +198,7 @@ int FFGET_setstream( FFGET_FILE *f, FILE *fi )
 	f->c = '\0';
 	f->FFEOF = 0;
 	f->FILEEND = 0;
+	f->last_block_read_from = -1;
 	return 0;
 }
 
@@ -229,6 +229,61 @@ int FFGET_feof( FFGET_FILE *f )
 }
 
 
+/*-----------------------------------------------------------------\
+ Function Name	: FFGET_seek
+ Returns Type	: int
+ 	----Parameter List
+	1. FFGET_FILE *f, 
+	2.  size_t offset , 
+ 	------------------
+ Exit Codes	: 
+ Side Effects	: 
+--------------------------------------------------------------------
+ Comments:
+		Seeks to 'offset' bytes from the first byte of the file.
+		Reloads the buffer block.
+ 
+--------------------------------------------------------------------
+ Changes:
+ 
+\------------------------------------------------------------------*/
+int FFGET_seek( FFGET_FILE *f, long offset, int whence )
+{
+	int result = 0;
+	fseek(f->f, offset, whence);
+	result = FFGET_getnewblock(f);
+	return result;
+}
+
+
+/*-----------------------------------------------------------------\
+ Function Name	: FFGET_tell
+ Returns Type	: size_t
+ 	----Parameter List
+	1. FFGET_FILE *f , 
+ 	------------------
+ Exit Codes	: 
+ Side Effects	: 
+--------------------------------------------------------------------
+ Comments:
+		Returns the position in the file that the current "file cursor"
+		is pointing to.
+
+ 
+--------------------------------------------------------------------
+ Changes:
+ 
+\------------------------------------------------------------------*/
+long FFGET_ftell( FFGET_FILE *f )
+{
+	long pos;
+
+	pos = f->last_block_read_from +(f->startpoint -f->buffer);
+
+	return pos;
+
+}
+
 /*------------------------------------------------------------------------
 Procedure:     FFGET_ungetc ID:1
 Purpose:       Pushes back into the buffer (effectively) a single character
