@@ -11,6 +11,7 @@ struct _FFGET_FILE
 	char buffer[_FFGET_BUFFER_MAX +1];
 	char *startpoint;
 	char *endpoint;
+	size_t last_block_read_from;
 	int FILEEND;
 	int FFEOF;
 	char c;
@@ -44,3 +45,5 @@ int FFGET_raw( FFGET_FILE *f, unsigned char *buffer, int max );
 int FFGET_feof( FFGET_FILE *f );
 int FFGET_getnewblock( FFGET_FILE *f );
 int FFGET_set_watch_SDL( int level );
+long FFGET_ftell( FFGET_FILE *f );
+int FFGET_fseek( FFGET_FILE *f, long offset, int whence );
