@@ -30,6 +30,7 @@ Description:   MIME-alter is an additional module in the mime.[ch] object, which
 #include "MIME_headers.h"
 #include "filename-filters.h"
 #include "mime_alter.h"
+#include "qpe.h"
 #include "logger.h"
 
 
@@ -77,6 +78,15 @@ struct AM_globals {
 	char *disclaimer_HTML;
 	int disclaimer_HTML_type;
 
+#ifdef ALTERMIME_PRETEXT
+	char *pretext_plain;
+	int pretext_plain_type;
+	char *pretext_HTML;
+	int pretext_HTML_type;
+
+	int pretext_insert;
+#endif
+
 	char *headerbuffer[ AM_HEADERBUFFER_MAX ];	// 100 lines for the header buffers
 
 	int headerbuffermax;
@@ -142,6 +152,14 @@ int AM_init( void )
 	glb.disclaimer_HTML=NULL;
 	glb.disclaimer_HTML_type=AM_DISCLAIMER_TYPE_NONE;
 
+#ifdef ALTERMIME_PRETEXT
+	glb.pretext_plain=NULL;
+	glb.pretext_plain_type=AM_PRETEXT_TYPE_NONE;
+	glb.pretext_HTML=NULL;
+	glb.pretext_HTML_type=AM_PRETEXT_TYPE_NONE;
+	glb.pretext_insert = 0;
+#endif
+
 	glb.headerbuffermax=0;
 
 	return 0;
@@ -195,6 +213,30 @@ int AM_set_force_for_bad_html( int level )
 }
 
 
+/*-----------------------------------------------------------------\
+ Function Name	: AM_set_pretext_insert
+ Returns Type	: int
+ 	----Parameter List
+	1. int level , 
+ 	------------------
+ Exit Codes	: 
+ Side Effects	: 
+--------------------------------------------------------------------
+ Comments:
+ 
+--------------------------------------------------------------------
+ Changes:
+ 
+\------------------------------------------------------------------*/
+#ifdef ALTERMIME_PRETEXT
+int AM_set_pretext_insert( int level )
+{
+	glb.pretext_insert = level;
+
+	return glb.pretext_insert;
+}
+#endif
+
 
 /*-----------------------------------------------------------------\
   Function Name	: AM_set_debug
@@ -468,6 +510,62 @@ char *AM_set_disclaimer_HTML( char *filename, int disclaimer_type )
 }
 
 
+/*-----------------------------------------------------------------\
+ Function Name	: *AM_set_pretext_plain
+ Returns Type	: char
+ 	----Parameter List
+	1. char *filename, 
+	2.  int pretext_type , 
+ 	------------------
+ Exit Codes	: 
+ Side Effects	: 
+--------------------------------------------------------------------
+ Comments:
+ 
+--------------------------------------------------------------------
+ Changes:
+ 
+\------------------------------------------------------------------*/
+#ifdef ALTERMIME_PRETEXT
+int AM_set_pretext_plain( char *filename, int pretext_type )
+{
+
+	glb.pretext_plain = strdup( filename );
+	glb.pretext_plain_type = pretext_type;
+
+	return 0;
+}
+#endif
+
+
+
+
+/*-----------------------------------------------------------------\
+ Function Name	: *AM_set_pretext_HTML
+ Returns Type	: char
+ 	----Parameter List
+	1. char *filename, 
+	2.  int pretext_type , 
+ 	------------------
+ Exit Codes	: 
+ Side Effects	: 
+--------------------------------------------------------------------
+ Comments:
+ 
+--------------------------------------------------------------------
+ Changes:
+ 
+\------------------------------------------------------------------*/
+#ifdef ALTERMIME_PRETEXT
+int AM_set_pretext_HTML( char *filename, int pretext_type )
+{
+	glb.pretext_HTML = strdup( filename );
+	glb.pretext_HTML_type = pretext_type;
+
+	return 0;
+}
+#endif
+
 /*------------------------------------------------------------------------
 Procedure:     AM_set_nullifyall ID:1
 Purpose:
@@ -689,17 +787,6 @@ int AM_read_headers(struct AM_disclaimer_details *dd, FFGET_FILE *f, FILE *newf
 	if (strlen(hinfo.filename) > 0) dd->isfile = 1;
 	if (strlen(hinfo.name) > 0) dd->isfile = 1;
 
-	/**
-	 ** list here any content types you don't want to 
-	 ** have disclaimers inserted into
-	 **/
-	switch (hinfo.content_type) {
-		case _CTYPE_MULTIPART_REPORT:
-		case _CTYPE_RFC822:
-			dd->isfile = 1;
-			break;
-	}
-
 	/**
 	 ** list here any content transfer encodings you
 	 ** don't want to have disclaimers inserted into
@@ -712,6 +799,13 @@ int AM_read_headers(struct AM_disclaimer_details *dd, FFGET_FILE *f, FILE *newf
 			break;
 	}
 
+	switch (hinfo.content_type) {
+		case _CTYPE_MULTIPART_REPORT:
+		case _CTYPE_RFC822:
+			dd->isfile = 1;
+			break;
+	}
+
 	/** If our content-type is of HTML, then we'll mark that we're in a HTML
 	 **		section.  NOTE - this doesn't mean that the section isn't a file!
 	 **		so, we have to check later on to see that dd.isfile == 0 still.
@@ -778,6 +872,59 @@ int AM_disclaimer_load_text( char *fname, char **textptr )
 
 
 
+
+int AM_disclaimer_html_perform_insertion( struct AM_disclaimer_details *dd, FFGET_FILE *f, FILE *newf )
+{
+	  if (dd->content_encoding == _CTRANS_ENCODING_QP) {
+		  char *qp_data;
+		  size_t qp_data_size;
+		  char *data_to_use;
+
+		  if (dd->disclaimer_text_HTML == NULL) data_to_use = dd->disclaimer_text_plain;
+		  else data_to_use = dd->disclaimer_text_HTML;
+
+			qp_data_size = strlen(data_to_use) *3 +1;
+			qp_data = malloc( qp_data_size *sizeof(char));
+			if (qp_data == NULL) {
+				LOGGER_log("%s:%d:AM_disclaimer_html_perform_insertion:DEBUG: Error trying to allocate %d bytes of memory for QP encoded disclaimer",FL, qp_data_size);
+				return -1;
+			}
+			qp_encode( qp_data, qp_data_size, data_to_use, strlen(data_to_use));
+			DAM LOGGER_log("%s:%d:AM_disclaimer_html_perform_insertion:DEBUG: Inserting QP encoded disclaimer",FL);
+			if (dd->disclaimer_text_HTML == NULL) {
+				fprintf(newf,"<br><pre>=\r\n");
+				AM_ntorn(qp_data, newf);
+				fprintf(newf,"</pre><br>=\r\n");
+			} else {
+			  fprintf(newf,"<br>=\r\n");
+				AM_ntorn(qp_data, newf);
+			  fprintf(newf,"<br>=\r\n");
+			}
+
+			/** cleanup **/
+			if (qp_data) free(qp_data);
+
+		  /** Quoted printable email segment, so we have to encode everything **/
+
+	  } else {
+
+		  if (dd->disclaimer_text_HTML == NULL)
+		  {
+			  fprintf(newf,"<br><pre>\r\n");
+			  AM_ntorn(dd->disclaimer_text_plain, newf);
+			  fprintf(newf,"</pre><br>\r\n");
+		  }
+		  else
+		  {
+			  fprintf(newf,"<br>\r\n");
+			  AM_ntorn(dd->disclaimer_text_HTML, newf);
+			  fprintf(newf,"<br>\r\n");
+		  }
+	  } /** quotedprintable / non encoding selector **/
+
+	  return 0;
+}
+
 /*------------------------------------------------------------------------
 Procedure:     AM_add_disclaimer_insert_html ID:1
 Purpose:       Inserts a disclaimer with <PRE> tagging into a HTML block of text just prior to the
@@ -820,18 +967,8 @@ int AM_add_disclaimer_insert_html( 	struct AM_disclaimer_details *dd, FFGET_FILE
 
            dd->html_inserted = 1;
 
-           if (dd->disclaimer_text_HTML == NULL)
-           {
-              fprintf(newf,"<br><pre>\r\n");
-				  AM_ntorn(dd->disclaimer_text_plain, newf);
-				  fprintf(newf,"</pre><br>\r\n");
-           }
-           else
-           {
-              fprintf(newf,"<br>\r\n");
-				  AM_ntorn(dd->disclaimer_text_HTML, newf);
-				  fprintf(newf,"<br>\r\n");
-           }
+				AM_disclaimer_html_perform_insertion( dd, f, newf );
+
         }
 
         // write the boundary line
@@ -852,16 +989,16 @@ int AM_add_disclaimer_insert_html( 	struct AM_disclaimer_details *dd, FFGET_FILE
 
 		// Look for either a BODY or HTML closing tag.
 
-		prebody = strstr(lline,"</body>");
-		if (!prebody) prebody = strstr(lline,"</html>");
+		prebody = strstr(lline,"</body");
+		if (!prebody) prebody = strstr(lline,"</html");
 
-		DAM LOGGER_log("%s:%d:AM_add_disclaimer_insert_html:DEBUG: prebody = %s",FL,prebody);
 
 		// If we found one of the tags, then insert our disclaimer aruond about
 		// here.
 
 		if (prebody != NULL)
 		{
+			DAM LOGGER_log("%s:%d:AM_add_disclaimer_insert_html:DEBUG: prebody = %s",FL,prebody);
 			DAM LOGGER_log("%s:%d:AM_add_disclaimer_insert_html:DEBUG: Inserting html-body disclaimer",FL);
 
 			dd->html_inserted = 1;
@@ -873,24 +1010,13 @@ int AM_add_disclaimer_insert_html( 	struct AM_disclaimer_details *dd, FFGET_FILE
 
 			// save to file the first part of the line segment
 
-			if ( dd->disclaimer_text_HTML == NULL )
-			{
-				fprintf(newf,"%s<BR><PRE>\r\n",line);
-				AM_ntorn(dd->disclaimer_text_plain, newf);
-				fprintf(newf,"</pre><BR>\r\n");
-			} else {
-				// replaced 20030611:0837:PLD				fprintf(newf,"%s<BR>", dd->disclaimer_text_HTML );
-				fprintf(newf, "%s<BR>\r\n",line);
-				AM_ntorn(dd->disclaimer_text_HTML, newf);
-				fprintf(newf,"<BR>\r\n");
-			}
-
+			fprintf(newf,"%s\r\n",line);
+			AM_disclaimer_html_perform_insertion( dd, f, newf );
 			fprintf(newf,"<%s",(tmpbody+1));
 
 			break;
-		}
-		else
-		{
+
+		} else {
 			fprintf(newf,"%s",line);
 		}
 
@@ -941,6 +1067,39 @@ int AM_add_disclaimer_cleanup( FILE *mp, FILE *newf, char *mpacktmp, char *mpack
 	return 0;
 }
 
+/*-----------------------------------------------------------------\
+  Function Name	: AM_add_disclaimer_flush
+  Returns Type	: int
+  ----Parameter List
+  1. FFGET_FILE *f, 
+  2. FILE *newf, 
+  ------------------
+  Exit Codes	: 
+  Side Effects	: 
+  --------------------------------------------------------------------
+Comments:
+
+--------------------------------------------------------------------
+Changes:
+
+\------------------------------------------------------------------*/
+int AM_add_disclaimer_flush( FFGET_FILE *f, FILE *newf )
+{
+	char line[AM_1K_BUFFER_SIZE+1]="";
+	
+	if ( ! FFGET_feof( f ) )
+	{
+		DAM LOGGER_log("%s:%d:AM_add_disclaimer_flush:DEBUG: Appending remaining of file",FL);
+		while (FFGET_fgets(line, AM_1K_BUFFER_SIZE, f))
+		{
+			fprintf(newf,"%s",line);
+		}
+		DAM LOGGER_log("%s:%d:AM_add_disclaimer_flush:DEBUG: Done appending.",FL);
+	}
+
+	return 0;
+}
+
 
 /*-----------------------------------------------------------------\
   Function Name	: AM_read_to_boundary
@@ -1178,7 +1337,30 @@ int AM_insert_disclaimer_into_segment( FFGET_FILE *f, FILE *newf, struct AM_disc
 
 		DAM LOGGER_log("%s:%d:AM_insert_disclaimer_into_segment:DEBUG: Inserting TEXT disclaimer (%s)\n", FL, dd->disclaimer_text_plain);
 
-		fprintf( newf, "%s\n", dd->disclaimer_text_plain );
+		if (dd->content_encoding == _CTRANS_ENCODING_QP)
+		{
+			/** convert the disclaimer into QP **/
+			char *qp_data;
+			size_t qp_data_size;
+
+			qp_data_size = strlen(dd->disclaimer_text_plain) *3 +1;
+			qp_data = malloc( qp_data_size *sizeof(char));
+			if (qp_data == NULL) {
+				LOGGER_log("%s:%d:AM_insert_disclaimer_into_segment:DEBUG: Error trying to allocate %d bytes of memory for QP encoded disclaimer",FL, qp_data_size);
+				return -1;
+			}
+			qp_encode( qp_data, qp_data_size, dd->disclaimer_text_plain, strlen(dd->disclaimer_text_plain));
+			DAM LOGGER_log("%s:%d:AM_insert_disclaimer_into_segment:DEBUG: Inserting QP encoded disclaimer",FL);
+			fprintf( newf, "%s\n", qp_data );
+
+			/** cleanup **/
+			if (qp_data) free(qp_data);
+
+		}  else {
+			/** Normal plain-text insertion **/
+			fprintf( newf, "%s\n", dd->disclaimer_text_plain );
+		}
+
 		dd->text_inserted = 1; /** Our disclaimer has been inserted, so set the flag **/
 		insert_success = 1;
 		DAM LOGGER_log("%s:%d:AM_insert_disclaimer_into_segment:DEBUG: TEXT disclaimer is inserted, now flushing file output",FL);
@@ -1198,6 +1380,14 @@ int AM_insert_disclaimer_into_segment( FFGET_FILE *f, FILE *newf, struct AM_disc
 
 	// Add in the HTML disclaimer (if wanted)
 
+	DAM LOGGER_log("%s:%d:AM_insert_disclaimer_into_segment:DEBUG: glb.HTML_too=%d, dd->html_inserted=%d, dd->content_type=%d, dd->isfile=%d"
+			,FL
+			,glb.HTML_too
+			,dd->html_inserted
+			,dd->content_type
+			,dd->isfile
+			);
+
 	if ( ( glb.HTML_too ) \
 			&& ( 0 == dd->html_inserted ) \
 			&& ( dd->content_type == _CTYPE_TEXT_HTML )\
@@ -1215,7 +1405,7 @@ int AM_insert_disclaimer_into_segment( FFGET_FILE *f, FILE *newf, struct AM_disc
 			insert_success = 1;
 		}
 	} else {
-		DAM LOGGER_log("%s:%d:AM_add_disclaimer_insert_html:DEBUG: Conditions not right to insert HTML disclaimer",FL);
+		DAM LOGGER_log("%s:%d:AM_insert_disclaimer_into_segment:DEBUG: Conditions not right to insert HTML disclaimer",FL);
 	}
 
 	/** If we weren't able to insert a disclaimer, then read through to the end of the segment **/
@@ -1269,8 +1459,8 @@ int AM_add_disclaimer( char *mpackname )
 
 	/* Initialise the Boundary-stack globals */
 	BS_init();
-	/* Allow only ONE (1) boundary to be stored */
-	BS_set_hold_limit(1);
+	/* Allow 10 boundaries to be stored */
+	BS_set_hold_limit(10);
 
 	// Test file statuses... and hop out if we had troubles with either
 	//
@@ -1354,6 +1544,11 @@ int AM_add_disclaimer( char *mpackname )
 	//
 	if (dd.content_encoding == _CTRANS_ENCODING_B64)
 	{
+		AM_add_disclaimer_flush( &f, newf );
+		/** 20050626-0111: Patch to remove tmp file on failure supplied by Carlos Velasco **/
+		fclose(newf);
+		remove(mpacktmp);
+		/** endpatch **/
 		VAM LOGGER_log("Email is BASE64 encoded, we cannot insert into these emails\n");
 		return AM_RETURN_B64_ENCODED_EMAIL;
 	}
@@ -1363,6 +1558,7 @@ int AM_add_disclaimer( char *mpackname )
 	//
 	if ((AM_RETURN_SIGNED_EMAIL == result)||(-1 == result))
 	{
+		AM_add_disclaimer_flush( &f, newf );
 		fclose(mp);
 		fclose(newf);
 		VAM LOGGER_log("Email is signed with a crypto, we do not alter these emails\n");
@@ -1433,22 +1629,57 @@ int AM_add_disclaimer( char *mpackname )
 			// Read SEGMENT 1 headers - and attempt to insert disclaimer
 			if (AM_read_headers( &dd, &f, newf ) == 0)
 			{
+				/** Did we stumble on a boundary, and was it a non-RFC822 form 
+				  ** (ie, not likely a forwarded email ) **/
+				if ((dd.boundary_found == 1)&&(dd.content_type != _CTYPE_RFC822))
+				{
+					DAM LOGGER_log("%s:%d:AM_add_disclaimer:DEBUG: Located a new non-RFC822 boundary, adding to list\n",FL);
+				}
+
+
 				DAM LOGGER_log("%s:%d:AM_add_disclaimer:DEBUG: inserting into SEGMENT 1",FL);
 				AM_insert_disclaimer_into_segment( &f, newf, &dd );
 				DAM LOGGER_log("%s:%d:AM_add_disclaimer:DEBUG: Finished inserting into SEGMENT 1",FL);
 			} else {
-				LOGGER_log("%s:%d:AM_add_disclaimer: Cannot read headers for SEGMENT 1",FL);
+				DAM LOGGER_log("%s:%d:AM_add_disclaimer: Cannot read headers for SEGMENT 1",FL);
 			}
 
 
 			// Read SEGMENT 2 headers - and attempt to insert disclaimer
 			if (AM_read_headers( &dd, &f, newf ) == 0)
 			{
+				/** Did we stumble on a boundary, and was it a non-RFC822 form 
+				  ** (ie, not likely a forwarded email ) **/
+				if ((dd.boundary_found == 1)&&(dd.content_type != _CTYPE_RFC822))
+				{
+					DAM LOGGER_log("%s:%d:AM_add_disclaimer:DEBUG: Located a new non-RFC822 boundary, adding to list\n",FL);
+				}
+
 				DAM LOGGER_log("%s:%d:AM_add_disclaimer:DEBUG: inserting into SEGMENT 2",FL);
 				AM_insert_disclaimer_into_segment( &f, newf, &dd );
 				DAM LOGGER_log("%s:%d:AM_add_disclaimer:DEBUG: Finished inserting into SEGMENT 2",FL);
 			} else {
-				LOGGER_log("%s:%d:AM_add_disclaimer: Cannot read headers for SEGMENT 2",FL);
+				DAM LOGGER_log("%s:%d:AM_add_disclaimer: Cannot read headers for SEGMENT 2",FL);
+			}
+
+
+
+			// Read SEGMENT 3 headers - and attempt to insert disclaimer, typically we do this
+			//		to handle TXT + HTML combo emails
+			if (AM_read_headers( &dd, &f, newf ) == 0)
+			{
+				/** Did we stumble on a boundary, and was it a non-RFC822 form 
+				  ** (ie, not likely a forwarded email ) **/
+				if ((dd.boundary_found == 1)&&(dd.content_type != _CTYPE_RFC822))
+				{
+					DAM LOGGER_log("%s:%d:AM_add_disclaimer:DEBUG: Located a new non-RFC822 boundary, adding to list\n",FL);
+				}
+
+				DAM LOGGER_log("%s:%d:AM_add_disclaimer:DEBUG: inserting into SEGMENT 3",FL);
+				AM_insert_disclaimer_into_segment( &f, newf, &dd );
+				DAM LOGGER_log("%s:%d:AM_add_disclaimer:DEBUG: Finished inserting into SEGMENT 3",FL);
+			} else {
+				DAM LOGGER_log("%s:%d:AM_add_disclaimer: Cannot read headers for SEGMENT 3",FL);
 			}
 
 		} while (0);
@@ -1456,16 +1687,7 @@ int AM_add_disclaimer( char *mpackname )
 
 
 	// Clean up any remaining lines in the email
-	if ( ! FFGET_feof( &f ) )
-	{
-		DAM LOGGER_log("%s:%d:AM_add_disclaimer:DEBUG: Appending remaining of file",FL);
-		while (FFGET_fgets(line, AM_1K_BUFFER_SIZE, &f))
-		{
-			//DAM LOGGER_log("%s:%d:AM_add_disclaimer:DEBUG: %d",FL,strlen(line));
-			fprintf(newf,"%s",line);
-		}
-		DAM LOGGER_log("%s:%d:AM_add_disclaimer:DEBUG: Done appending.",FL);
-	}
+	AM_add_disclaimer_flush( &f, newf );
 
 
 	VAM LOGGER_log("Done.\n");
@@ -1646,15 +1868,19 @@ int AM_nullify_attachment_recurse( struct MIMEH_header_info *hinfo, FFGET_FILE *
 
 		MIMEH_set_doubleCR_save(0);
 		MIMEH_set_header_longsearch(glb.header_long_search);
-		result = MIMEH_headers_get( hinfo, f );
 		MIMEH_set_doubleCR_save(1);
+
+		DAM LOGGER_log("%s:%d:AM_nullify_attachment_recurse:DEBUG: Reading headers... Iteration %d",FL);
+		result = MIMEH_headers_get( hinfo, f );
 		if (result != 0)
 		{ 
+			DAM LOGGER_log("%s:%d:AM_nullify_attachment_recurse:DEBUG: Failure during header read (EOF?), exiting loop",FL);
 			break;
 		}
 
 		/** If we detect a signed message and we're not explicitly wanting to alter them, then exit **/
 		if ((hinfo->content_type == _CTYPE_MULTIPART_SIGNED)&&(glb.alter_signed==0)) {
+			DAM LOGGER_log("%s:%d:AM_nullify_attachment_recurse:DEBUG: Message is signed, exiting",FL);
 			return 0;
 		}
 
@@ -1802,7 +2028,7 @@ int AM_nullify_attachment( char *mpackname, char *attachmentname )
 
 	// Nullifying an attachment can sometimes be a little bit tricky, we have to
 	//		dig down into the nesting of the MIME email and keep tabs on our
-	//		boundaries (via a Boundary-stack).  This all requires abot as much work
+	//		boundaries (via a Boundary-stack).  This all requires about as much work
 	//		as ripMIME just to get rid of one attachment.
 	//
 	// Additionally - we require some special functionality from MIME_headers, so
